#!/bin/bash

set -euo pipefail

pkgs=(
    build-essential
    curl
    gfortran
    git
    libssl-dev
    openssl
    pkg-config
)

apt update
apt install -y ${pkgs[@]}

# install rustup
curl --proto '=https' --tlsv1.2 -sSf https://sh.rustup.rs | sh -s -- --default-toolchain none -y

for version in ${RUST_V}; do
    # the last command will be the default
    rustup install ${version} --profile minimal
    rustup default ${version}
    # install LLVM tools needed for code coverage
    rustup component add llvm-tools
done

# install cargo-c needed for the CAPI
cargo install --locked cargo-c --version ${CARGOC_V}

# remove files generated by cargo
rm -r /usr/local/cargo/registry

# install LHAPDF
curl -L "https://lhapdf.hepforge.org/downloads/?f=LHAPDF-${LHAPDF_V}.tar.gz" | tar xzf -
cd LHAPDF-${LHAPDF_V}
./configure --disable-python
make -j V=1
make install
ldconfig
cd ..

# install PDF sets
for pdf in NNPDF31_nlo_as_0118_luxqed NNPDF40_nnlo_as_01180 NNPDF40_nlo_as_01180 NNPDF40_nlo_pch_as_01180; do
    curl "https://lhapdfsets.web.cern.ch/current/${pdf}.tar.gz" | tar xzf - -C /usr/local/share/LHAPDF
done

# install Tanjona's polarized PDF set
curl "https://data.nnpdf.science/pineappl/pdfs/240608-tr-pol-nlo-100.tar.gz" | tar xzf - -C /usr/local/share/LHAPDF

# install zlib; we need to link against it statically
curl "https://www.zlib.net/zlib-${ZLIB_V}.tar.gz" | tar xzf -
cd zlib-${ZLIB_V}
./configure --prefix=/usr/local
make -j
make install
ldconfig
cd ..

# install APPLgrid
curl -L "https://applgrid.hepforge.org/downloads?f=applgrid-${APPLGRID_V}.tgz" | tar xzf -
cd applgrid-${APPLGRID_V}
./configure --without-root
make -j
make install
ldconfig
mkdir -p ${APPL_IGRID_DIR}
cp src/*.h ${APPL_IGRID_DIR}
cd ..

# install fastNLO
curl "https://fastnlo.hepforge.org/code/v25/fastnlo_toolkit-${FASTNLO_V}.tar.gz" | tar xzf -
cd fastnlo_toolkit-${FASTNLO_V}
./configure --prefix=/usr/local/
make -j V=1
make install
ldconfig
cd ..
